#include "stc.h"
#include "usb.h"
#include "usb_desc.h"
#include "usb_req_class.h"
#include "keycode.h"
#include "uart.h"
#include "util.h"

LINECODING LineCoding;

BYTE bHidIdle;
BYTE bKeyCode = 0;
BYTE bKeyDebounce = 0;
BOOL fKeyOK;

WORD Key1_cnt;
WORD Key2_cnt;
WORD Key3_cnt;
BOOL Key1_Flag;
BOOL Key2_Flag;
BOOL Key3_Flag;

sbit KEY1 = P3^2;
sbit KEY2 = P3^3;
sbit KEY3 = P3^4;


sbit LED_NUM        = P2^3;
sbit LED_CAPS       = P2^2;
sbit LED_SCROLL     = P2^1;
sbit LED_TEST1      = P2^0;
unsigned char key_ss=0;
void usb_req_class()
{
    switch (Setup.bRequest)
    {
		case SET_LINE_CODING:
			usb_set_line_coding();
			break;
		case GET_LINE_CODING:
			usb_get_line_coding(); 
			break;
		case SET_CONTROL_LINE_STATE:
			usb_set_ctrl_line_state();
			break;
		case GET_REPORT:
			usb_get_report();
			break;
		case SET_REPORT:
			usb_set_report();
			break;
		case GET_IDLE:
			usb_get_idle();
			break;
		case SET_IDLE:
			usb_set_idle();
			break;
		case GET_PROTOCOL:
			usb_get_protocol();
			break;
		case SET_PROTOCOL:
			usb_set_protocol();
			break;		
		default:
			usb_setup_stall();
			return;
    }
}
void usb_set_line_coding()
{
    if ((DeviceState != DEVSTATE_CONFIGURED) ||
        (Setup.bmRequestType != (OUT_DIRECT | CLASS_REQUEST | INTERFACE_RECIPIENT)))
    {
        usb_setup_stall();
        return;
    }

    Ep0State.pData = (BYTE *)&LineCoding;
    Ep0State.wSize = Setup.wLength;

    usb_setup_out();
}

void usb_get_line_coding()
{
    if ((DeviceState != DEVSTATE_CONFIGURED) ||
        (Setup.bmRequestType != (IN_DIRECT | CLASS_REQUEST | INTERFACE_RECIPIENT)))
    {
        usb_setup_stall();
        return;
    }

    Ep0State.pData = (BYTE *)&LineCoding;
    Ep0State.wSize = Setup.wLength;

    usb_setup_in();
}

void usb_set_ctrl_line_state()
{
    if ((DeviceState != DEVSTATE_CONFIGURED) ||
        (Setup.bmRequestType != (OUT_DIRECT | CLASS_REQUEST | INTERFACE_RECIPIENT)))
    {
        usb_setup_stall();
        return;
    }

    usb_setup_status();
}
void usb_uart_settings()
{
    if (Setup.bRequest == SET_LINE_CODING)
    {
        LineCoding.bCharFormat = 0;
        LineCoding.bDataBits = 8;
        
        uart_set_parity(LineCoding.bParityType);
        uart_set_baud(reverse4(LineCoding.dwDTERate));
    }
}
void usb_get_report()
{
    if ((DeviceState != DEVSTATE_CONFIGURED) ||
        (Setup.bmRequestType != (IN_DIRECT | CLASS_REQUEST | INTERFACE_RECIPIENT)))
    {
        usb_setup_stall();
        return;
    }

    Ep0State.pData = UsbBuffer;
    Ep0State.wSize = Setup.wLength;

    usb_setup_in();
}

void usb_set_report()
{
    if ((DeviceState != DEVSTATE_CONFIGURED) ||
        (Setup.bmRequestType != (OUT_DIRECT | CLASS_REQUEST | INTERFACE_RECIPIENT)))
    {
        usb_setup_stall();
        return;
    }

    Ep0State.pData = UsbBuffer;
    Ep0State.wSize = Setup.wLength;

    usb_setup_out();
}

void usb_get_idle()
{
    if ((DeviceState != DEVSTATE_CONFIGURED) ||
        (Setup.bmRequestType != (IN_DIRECT | CLASS_REQUEST | INTERFACE_RECIPIENT)))
    {
        usb_setup_stall();
        return;
    }

    Ep0State.pData = &bHidIdle;
    Ep0State.wSize = 1;

    usb_setup_in();
}

void usb_set_idle()
{
    if ((DeviceState != DEVSTATE_CONFIGURED) ||
        (Setup.bmRequestType != (OUT_DIRECT | CLASS_REQUEST | INTERFACE_RECIPIENT)))
    {
        usb_setup_stall();
        return;
    }

    bHidIdle = Setup.wValueH;

    usb_setup_status();
}

void usb_get_protocol()
{
    Ep0State.pData = PACKET0;
    Ep0State.wSize = 1;

    usb_setup_in();
}

void usb_set_protocol()
{
    usb_setup_status();
}
BYTE usb_bulk_intr_out(BYTE *pData, BYTE ep)
{
    BYTE cnt;

    cnt = usb_read_fifo((BYTE)(FIFO0 + ep), pData);
    usb_write_reg(OUTCSR1, 0);

    return cnt;
}
void usb_class_out()
{
    BYTE led;
    
    if (usb_bulk_intr_out(UsbBuffer, 1) >= 1)
    {
//        P4M0 &= ~0x01;
//        P4M1 &= ~0x01;
//        P6M0 &= ~0xff;
//        P6M1 &= ~0xff;
//        P40 = 0;
        
        led = UsbBuffer[1];
        LED_NUM = !(led & 0x01);
        LED_CAPS = !(led & 0x02);
        LED_SCROLL = !(led & 0x04);
    }
}

static void delay()
{
    int i;
    
    for (i = 0; i < 3000; i++)
    {
        _nop_();
        _nop_();
        _nop_();
        _nop_();
    }
}

void usb_class_in()
{
    BYTE key[9];
    BYTE i;
    
    if (DeviceState != DEVSTATE_CONFIGURED)  //USBûɣֱ˳
        return;

    if (!UsbInBusy && fKeyOK)  //жUSBǷУԼǷа
    {
        fKeyOK = 0;
P25 = ~P25;
        key[0] = 1;  //ͻ
        key[1] = 0;
        key[2] = 0;
        key[3] = 0;
        key[4] = 0;
        key[5] = 0;
        key[6] = 0;
        key[7] = 0;
				key[8] = 0;

        switch (bKeyCode)
        {
        case 0x0e:  key[3] = 0x1e; break;  //0x1e -> "1"
        case 0x0d:  key[3] = 0x1f; break;  //0x1f -> "2"
        case 0x0b:  key[3] = 0x20; break;  //0x20 -> "3"
        case 0x07:  key[3] = 0x21; break;  //0x21 -> "4"
        }
        
        IE2 &= ~0x80;   //EUSB = 0;
        UsbInBusy = 1;
        usb_write_reg(INDEX, 1);
        for (i=0; i<9; i++)
        {
            usb_write_reg(FIFO1, key[i]);  //Ͱ
        }
        usb_write_reg(INCSR1, INIPRDY);
        IE2 |= 0x80;    //EUSB = 1;
    }
}

//void usb_class_in()
//{
//    static BYTE id = 0;
//    BYTE key[9];
//    BYTE i;

//    if (DeviceState != DEVSTATE_CONFIGURED)
//        return;

//    if (!UsbInBusy && fKeyOK)
//    {
//        fKeyOK = 0;

//        key[0] = 0;
//        key[1] = 0;
//        key[2] = 0;
//        key[3] = 0;
//        key[4] = 0;
//        key[5] = 0;
//        key[6] = 0;
//        key[7] = 0; 
//        key[8] = 0;

//        switch (bKeyCode)
//        {
//        case 0x01:
//            id = 1;
//            key[3] = 0x4b;
//            break;
//        case 0x02:
//            id = 1;
//            key[3] = 0x4e;
//            break;
//        case 0x03:
//            id = 1;
//						key[3] = 0x1e;
//            //key[1] = 0x08;
//            break;
////        case 0xf7:
////            id = 1;
////            key[3] = ID_KB_B;
////            break;
//					
//					
////        case 0x01:
////            id = 2;
////            key[1] = ID_MK_VolumeInc;
////            key[2] = ID_MK_VolumeInc >> 8;
////            break;
////        case 0x02:
////            id = 2;
////            key[1] = ID_MK_VolumeDec;
////            key[2] = ID_MK_VolumeDec >> 8;
////            break;
////        case 0x03:
////            id = 2;
////            key[1] = ID_MK_Mute;
////            break;				
//				

////        case 0x02:
////            id = 3;
////            key[1] = ID_KB_Wakeup;
////            break;
////        case 0x03:
////            id = 3;
////            key[1] = ID_KB_Power;
////            break;
//        }
//        key[0] = id;
//        
//        IE2 &= ~0x80;
//        if (usb_read_reg(POWER) & 0x02)
//        {
//            LED_TEST1 = 0;
//            usb_write_reg(POWER, 5);
//            delay();
//            usb_write_reg(POWER, 1);
//        }
//        
//        UsbInBusy = 1;
//        usb_write_reg(INDEX, 1);
//        for (i=0; i<9; i++)
//        {
//            usb_write_reg(FIFO1, key[i]);
//        }
//        usb_write_reg(INCSR1, INIPRDY);
//		key_ss=1;
//        IE2 |= 0x80;
//    }
//}
//void usb_class_in()
//{
//    static BYTE id = 0;	 
//    BYTE key[8];
//    BYTE i;
//    
//    if (DeviceState != DEVSTATE_CONFIGURED)
//        return;

//    if (!UsbInBusy && fKeyOK)
//    {
//        fKeyOK = 0;

//        key[0] = 1;
//        key[1] = 0;
//        key[2] = 0;
//        key[3] = 0;
//        key[4] = 0;
//        key[5] = 0;
//        key[6] = 0;
//        key[7] = 0;
////	      key[8] = 0;		

//        switch (bKeyCode)
//        {
////          case 0x01:id=3;  key[1] = ID_KB_Power; break; //P20 key[0] 0X08  0x80 WINʼ
////          case 0x02:id=3;  key[1] = ID_KB_Sleep; break; //P21
//          case 0x01:
//						   id=1;
//					     key[3] = ID_KP_1; 
//					     break; 
//          case 0x02:
//						   id=1;
//					     key[3] = ID_KP_2;
//     					 break; 				
////          case 0x03:  key[2] = 0x20; break; //P32+P33
//				
//        }
//         key[0] = id;
//        IE2 &= ~0x80;   //EUSB = 0;
//        UsbInBusy = 1;
//        usb_write_reg(INDEX, 1);
//        for (i=0; i<8; i++)
//        {
//            usb_write_reg(FIFO1, key[i]);
//        }
//        usb_write_reg(INCSR1, INIPRDY);
//        IE2 |= 0x80;    //EUSB = 1;
//    }
//}


//void scan_key()
//{

//	

//	if(!KEY1)
//	{
//		if(!Key1_Flag)
//		{
//			Key1_cnt++;
//			if(Key1_cnt >= 50)  //50ms
//			{
//				Key1_Flag = 1;  //ð״ֹ̬ظ
//				bKeyCode |= 1;
//                fKeyOK = 1;
//			}
//		}
//	}
//	else
//	{
//		Key1_cnt = 0;
//        if(Key1_Flag)
//        {
//            bKeyCode &= ~1;
//            fKeyOK = 1;
//            Key1_Flag = 0;
//        }
//	}

//	if(!KEY2)
//	{
//		if(!Key2_Flag)
//		{
//			Key2_cnt++;
//			if(Key2_cnt >= 50)  //50ms
//			{
//				Key2_Flag = 1;  //ð״ֹ̬ظ
//				bKeyCode |= 2;
//        fKeyOK = 1;
//			}
//		}
//	}
//	else
//	{
//		Key2_cnt = 0;
//        if(Key2_Flag)
//        {
//            bKeyCode &= ~2;
//            fKeyOK = 1;
//            Key2_Flag = 0;
//        }
//	}
//	if(!KEY3)
//	{
//		if(!Key3_Flag)
//		{
//			Key3_cnt++;
//			if(Key3_cnt >= 50)  //50ms
//			{
//				Key3_Flag = 1;  //ð״ֹ̬ظ
//				bKeyCode |= 3;
//        fKeyOK = 1;
//			}
//		}
//	}
//	else
//	{
//		Key3_cnt = 0;
//        if(Key3_Flag)
//        {
//            bKeyCode &= ~3;
//            fKeyOK = 1;
//            Key3_Flag = 0;
//        }
//	}		
//}
void scan_key()
{
    BYTE key;
    
    key = 0;
    P3 |= 0x3c;
    _nop_();
    _nop_();
    key = (P3 & 0x3c) >> 2;  //ȡP3.2~P3.5״̬
    
    if (key != bKeyCode)
    {
        bKeyCode = key;
        bKeyDebounce = 20;  //÷ʱ 20 * 1ms = 20ms
    }
    else
    {
        if (bKeyDebounce)
        {
            bKeyDebounce--;
            if (bKeyDebounce == 0)
            {
                fKeyOK = 1;  //⵽״̬ͱ仯
            }
        }
    }
    
}

//void scan_key()
//{
//    BYTE key;
//    
//    P0M0 = 0x00;
//    P0M1 = 0x00;
//    
//    key = 0;
//    P0 = 0xff;
//    P06 = 0;
//    _nop_();
//    _nop_();
//    key |= P0 & 0x0f;
//    P06 = 1;
//    P07 = 0;
//    _nop_();
//    _nop_();
//    key |= (P0 & 0x0f) << 4;
//    P07 = 1;
//    
//    if (key != bKeyCode)
//    {
//        bKeyCode = key;
//        bKeyDebounce = 20;
//    }
//    else
//    {
//        if (bKeyDebounce)
//        {
//            bKeyDebounce--;
//            if (bKeyDebounce == 0)
//            {
//                fKeyOK = 1;
//            }
//        }
//    }
//    
//}

